from pydantic import BaseModel, Field
from dataclasses import dataclass

from typing import Type
import httpx

from argos.schemas import Task

# XXX We could name this Result, but is it could overlap with schemas.Result.
# Need to better define the naming around this.
@dataclass
class Response:
    status: str
    context: dict


class BaseExpectedValue(BaseModel):
    expected: str

    def get_converted(self):
        return self.expected


class ExpectedIntValue(BaseExpectedValue):
    def get_converted(self):
        return int(self.expected)


class ExpectedStringValue(BaseExpectedValue):
    pass


class CheckNotFound(Exception):
    pass


class BaseCheck:
    config: str
    expected_cls: Type[BaseExpectedValue] = None

    _registry = []

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        cls._registry.append(cls)

    @classmethod
    def get_registered_checks(cls):
        return {c.config: c for c in cls._registry}

    @classmethod
    def get_registered_check(cls, name):
        check = cls.get_registered_checks().get(name)
        if not check:
            raise CheckNotFound(name)
        return check

    def response(self, passed, **kwargs) -> Response:
        status = "success" if passed else "failure"
        return Response(status, kwargs)

    @property
    def expected(self):
        return self.expected_cls(expected=self.task.expected).get_converted()

    def __init__(self, client: httpx.AsyncClient, task: Task):
        self.client = client
        self.task = task


def get_check_by_name(name):
    return BaseCheck.get_registered_check(name)