from argos.logging import logger
from argos.checks.base import BaseCheck, Response, ExpectedIntValue, ExpectedStringValue
import ssl
import time

from datetime import datetime
from OpenSSL import crypto


class HTTPStatus(BaseCheck):
    config = "status-is"
    expected_cls = ExpectedIntValue

    async def run(self) -> dict:
        # XXX Get the method from the task
        task = self.task
        response = await self.client.request(method="get", url=task.url)
        logger.error(f"{response.status_code=}, {self.expected=}")
        return self.response(
            response.status_code == self.expected,
            expected=self.expected,
            retrieved=response.status_code,
        )


class HTTPBodyContains(BaseCheck):
    config = "body-contains"
    expected_cls = ExpectedStringValue

    async def run(self) -> dict:
        response = await self.client.request(method="get", url=self.task.url)
        return self.response(self.expected in response.text)


class SSLCertificateExpiration(BaseCheck):
    config = "ssl-certificate-expiration"
    expected_cls = ExpectedStringValue

    async def run(self):
        response = await self.client.get(self.task.url)
        if response.is_error:
            raise

        conn = self.client.transport.get_connection_info(self.task.url)
        cert = ssl.DER_cert_to_PEM_cert(conn.raw_certificates[0])

        x509 = crypto.load_certificate(crypto.FILETYPE_PEM, cert)
        not_after = x509.get_notAfter().decode("utf-8")
        not_after = datetime.strptime(not_after, "%Y%m%d%H%M%SZ")

        now = time.time()
        if time.mktime(not_after.timetuple()) < now:
            expired = True
        else:
            expired = False

        return self.response(
            expired == False, expected=now, retrieved=not_after.timetuple()
        )
