from pydantic import BaseModel
from datetime import datetime
from typing import Literal
import traceback

# XXX Refactor using SQLModel to avoid duplication of model data

class Task(BaseModel):
    id : int
    url: str
    domain: str
    check: str
    expected: str
    selected_at: datetime | None
    selected_by : str | None

    class Config:
        from_attributes = True


class SerializableException(BaseModel):
    error_message: str
    error_type: str
    error_details: str

    @staticmethod
    def from_exception(e: BaseException):
        return SerializableException(
            error_message=str(e),
            error_type=str(type(e).__name__),
            error_details=traceback.format_exc()
        )

class ClientResult(BaseModel):
    task : int
    status : Literal["success", "failure", "error"]
    context: dict | SerializableException