from fastapi import Depends, FastAPI, HTTPException, Request
from sqlalchemy.orm import Session
from pydantic import BaseModel

from argos.server import queries, models
from argos.schemas import ClientResult, Task
from argos.schemas.config import from_yaml as get_schemas_from_yaml
from argos.server.database import SessionLocal, engine
from argos.logging import logger
from typing import List

models.Base.metadata.create_all(bind=engine)

app = FastAPI()


def get_db():
    db = SessionLocal()
    try:
        yield db
    finally:
        db.close()


@app.on_event("startup")
async def read_config_and_populate_db():
    # XXX Get filename from environment.
    config = get_schemas_from_yaml("config.yaml")

    db = SessionLocal()
    try:
        await queries.update_from_config(db, config)
    finally:
        db.close()


# XXX Get the default limit from the config
@app.get("/tasks", response_model=list[Task])
async def read_tasks(request: Request, limit: int = 20, db: Session = Depends(get_db)):
    tasks = await queries.list_tasks(db, client_id=request.client.host, limit=limit)
    return tasks


@app.post("/results", status_code=201)
async def create_result(results: List[ClientResult], db: Session = Depends(get_db)):
    db_results = []
    for client_result in results:
        db_results.append(await queries.create_result(db, client_result))
    db.commit()
    return {"result_ids": [r.id for r in db_results]}