from sqlalchemy.orm import Session
from sqlalchemy import exists

from argos import schemas
from argos.logging import logger
from argos.server.models import Task, Result

from urllib.parse import urljoin
from datetime import datetime


async def list_tasks(db: Session, client_id: str, limit: int = 100):
    """List tasks and mark them as selected"""
    tasks = db.query(Task).where(Task.selected_by == None).limit(limit).all()
    now = datetime.now()
    # XXX: Deactivated for now, as it simplifies testing.
    # for task in tasks:
    #     task.selected_at = now
    #     task.selected_by = client_id
    # db.commit()
    return tasks


async def create_result(db: Session, client_result: schemas.ClientResult):
    result = Result(
        submitted_at=datetime.now(),
        status=client_result.status,
        context=client_result.context,
    )
    db.add(result)
    return result


async def update_from_config(db: Session, config: schemas.Config):
    for website in config.websites:
        domain = str(website.domain)
        for p in website.paths:
            url = urljoin(domain, str(p.path))
            for check in p.checks:
                for check_key, expected in check.items():
                    # Check the db for already existing tasks.
                    existing_task = db.query(
                        exists().where(
                            Task.url == url
                            and Task.check == check_key
                            and Task.expected == expected
                        )
                    ).scalar()

                    if not existing_task:
                        task = Task(
                            domain=domain, url=url, check=check_key, expected=expected
                        )
                        logger.debug(f"Adding a new task in the db: {task=}")
                        db.add(task)
                    else:
                        logger.debug(
                            f"Skipping db task creation for {url=}, {check_key=}, {expected=}."
                        )
    db.commit()
